package tools;

import java.math.BigDecimal;
import cartago.*;

public class Env extends Artifact {

	private Display display;
	private boolean locked;
	private int nBarrierParticipants;

    void init(int nmax) {
    	this.defineObsProperty("numMsg",0);
    	display = new Display();
    	display.setVisible(true);
			locked = false;
			nBarrierParticipants = nmax;
    	System.out.println("Env ready.");
	}

    @OPERATION void printMsg(String msg){
    	String agentName = this.getCurrentOpAgentId().getAgentName();
    	display.addText("Message from "+agentName+": "+msg);
      ObsProperty prop = this.getObsProperty("numMsg");
    	prop.updateValue(prop.intValue()+1);
    	display.updateNumMsgField(prop.intValue());
    }

		@OPERATION void computePi(int numDigits, OpFeedbackParam<String> res){
			BigDecimal digits = CalcLib.computePiDigits(numDigits);
			res.set(digits.toPlainString());
		}

		@OPERATION void lock(){
    	await("notLocked");
    	locked = true;
    }

    @GUARD boolean notLocked(){
    	return !locked;
    }

    @OPERATION void unlock(){
    	locked = false;
    }

		@OPERATION void setupBarrier(int nParticipants){
			nBarrierParticipants = nParticipants;
		}

		@OPERATION void synch(){
			nBarrierParticipants--;
			await("allArrived");
		}

		@GUARD boolean allArrived(){
			return nBarrierParticipants == 0;
		}
}
